@@ -374,28 +374,29 @@ class DwdWeather(object):
             return datetime.utcnow() - latest
 
 
-    def query(self, station_id, hour):
+    def query(self, station_id, hour, recursion=0):
         """
         Get values from cache.
         station_id: Numeric station ID
         hour: datetime object
         """
-        sql = "SELECT * FROM measures WHERE station_id=? AND datetime=?"
-        c = self.db.cursor()
-        c.execute(sql, (station_id, hour.strftime("%Y%m%d%H")))
-        out = c.fetchone()
-        if out is None:
-            # cache miss
-            age = (datetime.utcnow() - hour).total_seconds() / 86400
-            if age < 365:
-                self.import_measures(station_id, latest=True)
-            elif age >= 365 and age <= 366:
-                self.import_measures(station_id, latest=True, historic=True)
-            else:
-                self.import_measures(station_id, historic=True)
-            return self.query(station_id, hour)
-        c.close()
-        return out
+        if recursion < 2 :
+            sql = "SELECT * FROM measures WHERE station_id=? AND datetime=?"
+            c = self.db.cursor()
+            c.execute(sql, (station_id, hour.strftime("%Y%m%d%H")))
+            out = c.fetchone()
+            if out is None:
+                # cache miss
+                age = (datetime.utcnow() - hour).total_seconds() / 86400
+                if age < 360:
+                    self.import_measures(station_id, latest=True)
+                elif age >= 360 and age <= 370:
+                    self.import_measures(station_id, latest=True, historic=True)
+                else:
+                    self.import_measures(station_id, historic=True)
+                return self.query(station_id, hour, recursion=(recursion + 1))
+            c.close()
+            return out
 
     def haversine_distance(self, origin, destination):
         lon1, lat1 = origin
@@ -415,7 +416,9 @@ class DwdWeather(object):
         Return list of dicts with all stations
         """
         out = []
-        sql = "SELECT * FROM stations WHERE date_end IS NULL"
+        sql = """SELECT * FROM stations
+            WHERE date_end IS NULL
+            ORDER BY station_id"""
         c = self.db.cursor()
         for row in c.execute(sql):
             out.append(row)
@@ -455,35 +458,115 @@ class DwdWeather(object):
                     "coordinates": [station["geo_lon"], station["geo_lat"]]
                 }
             })
-        return out
+        import json
+        return json.dumps(out)
+
+    def stations_csv(self, delimiter=","):
+        """
+        Return stations list as CSV
+        """
+        import csv
+        import StringIO
+        csvfile = StringIO.StringIO()
+        # assemble field list
+        headers = ["station_id", "date_start", "date_end",
+            "geo_lon", "geo_lat", "height", "name"]
+        writer = csv.writer(csvfile, delimiter=delimiter, quoting=csv.QUOTE_MINIMAL)
+        writer.writerow(headers)
+        stations = self.stations()
+        for station in stations:
+            row = []
+            for n in range(len(headers)):
+                val = station[headers[n]]
+                if val is None:
+                    val = ""
+                elif type(val) == int:
+                    val = str(val)
+                elif type(val) == float:
+                    val = "%.4f" % val
+                elif type(val) == unicode:
+                    val = val.encode("utf8")
+                row.append(val)
+            writer.writerow(row)
+        contents = csvfile.getvalue()
+        csvfile.close()
+        return contents
 
 if __name__ == "__main__":
+
+
+    def get_station(args):
+        dw = DwdWeather(user=args.user,
+            passwd=args.passwd,
+            cachepath=args.cachepath)
+        import json
+        print json.dumps(dw.nearest_station(lon=args.lon, lat=args.lat), indent=4)
+
+    def get_stations(args):
+        dw = DwdWeather(user=args.user,
+            passwd=args.passwd,
+            cachepath=args.cachepath)
+        output = ""
+        if args.type == "geojson":
+            output = dw.stations_geojson()
+        elif args.type == "csv":
+            output = dw.stations_csv()
+        elif args.type == "plain":
+            output = dw.stations_csv(delimiter="\t")
+        if args.output_path is None:
+            print output
+        else:
+            f = open(args.output_path, "wb")
+            f.write(output)
+            f.close()
+
+    def get_weather(args):
+        hour = datetime.strptime(str(args.hour), "%Y%m%d%H")
+        dw = DwdWeather(user=args.user,
+            passwd=args.passwd,
+            cachepath=args.cachepath)
+        import json
+        print json.dumps(dw.query(2667, hour), indent=4)
+
     import argparse
-    parser = argparse.ArgumentParser()
-    parser.add_argument("-u", dest="user",
-        help="DWD FTP user name")
-    parser.add_argument("-p", dest="passwd",
-        help="DWD FTP user password")
-    parser.add_argument("--cache", dest="cachepath",
-        help="Path to cache directory. Defaults to .dwd-weather in user's home dir.")
+    argparser = argparse.ArgumentParser(prog="dwdweather",
+        description="Get weather information for Germany.")
+    argparser.add_argument("-u", dest="user",
+        help="DWD FTP user name. If not set, environment variable DWDUSER is used.",
+        default=os.environ.get("DWDUSER"))
+    argparser.add_argument("-p", dest="passwd",
+        help="DWD FTP user password. If not set, environment variable DWDPASS is used.",
+        default=os.environ.get("DWDPASS"))
+    argparser.add_argument("-c", dest="cachepath",
+        help="Path to cache directory. Defaults to .dwd-weather in user's home dir.",
+        default=os.path.expanduser("~") + os.sep + ".dwd-weather")
 
-    args = parser.parse_args()
-
-    dw = DwdWeather(user=args.user,
-        passwd=args.passwd,
-        cachepath=args.cachepath)
-
-    print("Station close to Cologne:")
-    print(dw.nearest_station(lon=7, lat=51))
+    subparsers = argparser.add_subparsers(title="Actions", help="Main client actions.")
     
-    print("Latest entry age:")
-    print dw.get_data_age()
-    
-    import json
-    print("Weather at Cologne/Bonn airport at 2012-01-12 12:00 UTC:")
-    print json.dumps(dw.query(2667, datetime(2012, 1, 12, 12)), indent=4, sort_keys=True)
+    # station options
+    parser_station = subparsers.add_parser('station',
+        help='Find a station')
+    parser_station.set_defaults(func=get_station)
+    parser_station.add_argument("lon", type=float, choices=range(-180, 180),
+        help="Geographic longitude (x) component as float, e.g. 7.2")
+    parser_station.add_argument("lat", type=float, choices=range(-90, 90),
+        help="Geographic latitude (y) component as float, e.g. 53.9")
 
-    print("Saving stations GeoJSON file as stations.geojson.")
-    fp = open("stations.geojson", "wb")
-    fp.write(json.dumps(dw.stations_geojson()))
-    fp.close()
+    # stations options
+    parser_stations = subparsers.add_parser('stations',
+        help='List or export stations')
+    parser_stations.set_defaults(func=get_stations)
+    parser_stations.add_argument("-t", "--type", dest="type",
+        choices=["geojson", "csv", "plain"], default="plain",
+        help="Export format")
+    parser_stations.add_argument("-f", "--file", type=str, dest="output_path",
+        help="Export file path. If not given, STDOUT is used.")
+    
+    # weather options
+    parser_weather = subparsers.add_parser('weather', help='Get weather data for a station and hour')
+    parser_weather.set_defaults(func=get_weather)
+    parser_weather.add_argument("station_id", type=int, help="Numeric ID of the station, e.g. 2667")
+    parser_weather.add_argument("hour", type=int, help="Time in the form of YYYYMMDDHH")
+
+    args = argparser.parse_args()
+    args.func(args)
